@@ -1,32 +1,38 @@
 (**************************************************************************)
 (*                                                                        *)
 (*     The Alt-ergo theorem prover                                        *)
-(*     Copyright (C) 2006-2008                                            *)
+(*     Copyright (C) 2006-2010                                            *)
 (*                                                                        *)
 (*     Sylvain Conchon                                                    *)
 (*     Evelyne Contejean                                                  *)
-(*     CNRS-LRI-Universite Paris Sud                                      *)
+(*     Stephane Lescuyer                                                  *)
+(*     Mohamed Iguernelala                                                *)
+(*     Alain Mebsout                                                      *)
+(*                                                                        *)
+(*     CNRS - INRIA - Universite Paris Sud                                *)
 (*                                                                        *)
 (*   This file is distributed under the terms of the CeCILL-C licence     *)
 (*                                                                        *)
 (**************************************************************************)
 
- open Format
+open Format
 open Hashcons
 
 module Sy = Symbols
+module T = Term
 
 type sort_var = A | B | C
     
 type tvar = { var : int ; sorte : sort_var }
 
-type 'a xterm = Var of tvar | Term of 'a
+type 'a xterm = Var of tvar | Alien of 'a
     
 type 'a alpha_term = {
   bv : 'a;
   sz : int;
 }
 
+
 type 'a simple_term_aux = 
   | Cte of bool 
   | Other of 'a xterm
@@ -34,24 +40,22 @@ type 'a simple_term_aux =
       
 type 'a simple_term = ('a simple_term_aux) alpha_term
 
+type 'a abstract =  ('a simple_term) list 
+    
+
+(* for the solver *)
+
 type solver_simple_term_aux = 
     | S_Cte of bool 
     | S_Var of tvar
 
 type solver_simple_term = solver_simple_term_aux alpha_term
 
-type 'a abstract =  ('a simple_term) list 
-    
+
 module type ALIEN = sig
-  type r
-  val make : Term.t -> r
-  val type_infos : r -> Ty.t
-  val extract : r -> (r abstract) option
+  include Sig.X
   val embed : r abstract -> r
-  val subst : r -> r -> r -> r
-  val leaves : r -> r list
-  val compare : r -> r -> int
-  val print : Format.formatter -> r -> unit
+  val extract : r -> (r abstract) option
 end
 
 module Make(X : ALIEN) = struct
@@ -63,26 +67,40 @@ module Make(X : ALIEN) = struct
 
   let is_mine_a _ = false
 
-  let is_mine_hs t = 
-    match Term.view t with
-	{Term.f = (Sy.Bitv _ | Sy.Binop Sy.At) } -> true
-      | {Term.f = s} -> Sy.equal s Sy.shat
+  let is_mine_symb = function
+    | Sy.Bitv _ | Sy.Binop Sy.At -> true
+    | s -> Sy.equal s Sy.shat
 
   let is_mine_type _ = assert false
 
-  let normal_form x = x
+  let existential = function
+      [{bv =Other (Var _)}] -> true
+    | _ -> false
+	
+  let embed r =
+    match X.extract r with
+      | None ->
+	  begin
+	    match X.type_info r with
+	      | Ty.Tbitv n -> [{bv = Other (Alien r) ; sz = n}]
+	      | _  -> assert false
+	  end
+      | Some b -> b
+
 
-  let embedding r = assert false
-      
   let err = err_formatter
 
   let compare_xterm xt1 xt2 = match xt1,xt2 with
     | Var v1, Var v2 -> 
-	let c1 = compare v1.sorte v2.sorte
-	in if c1 <> 0 then c1 else compare v1.var v2.var
-    | Term t1, Term t2 -> X.compare t1 t2
-    | Var v, Term t -> 1
-    | Term t, Var v -> -1
+	let c1 = compare v1.sorte v2.sorte in 
+	if c1 <> 0 then c1 
+	else -(compare v1.var v2.var)
+	  (* on inverse le signe : les variables les plus fraiches sont 
+	     les plus jeunes (petites)*)
+
+    | Alien t1, Alien t2 -> X.compare t1 t2
+    | Var v, Alien t -> 1
+    | Alien t, Var v -> -1
 
   let compare_simple_term st1 st2 = 
     if st1.sz <> st2.sz then st1.sz - st2.sz
@@ -130,7 +148,7 @@ module Make(X : ALIEN) = struct
 	  
     and term = term_aux alpha_term
 
-    (**)
+    (** **)
     let rec alpha t = match t.bv with
       |I_Cte _ -> [t]
       |I_Other _ -> [t]
@@ -154,7 +172,7 @@ module Make(X : ALIEN) = struct
 		@(alpha{sz = v.sz-i ; bv = I_Ext(v,i,v.sz-1)})
 	 end
 	   
-    (**)   
+    (** **)   
     let rec beta lt = 
       let simple_t st = match st.bv with
 	|I_Cte b -> {bv = Cte b ; sz = st.sz}
@@ -183,7 +201,7 @@ module Make(X : ALIEN) = struct
 	       |_ -> (simple_t s)::(beta (t::tl'))
 	   end
 	
-    (**)     
+    (** **)     
     let sigma term = beta (alpha term)
       
     let bitv_to_icomp =
@@ -203,24 +221,31 @@ module Make(X : ALIEN) = struct
       bitv_to_icomp (List.hd res) (List.tl res)
 
     let make t =
-      let rec make_rec t' = match Term.view t' with
-	| {Term.f = Sy.Bitv s } -> string_to_bitv s
-	| {Term.f = Sy.Binop Sy.At ; xs = [t1;t2] ; ty = Ty.Tbitv n} ->
-	    { bv = I_Comp (make_rec t1,make_rec t2) ; sz = n }
-	| {Term.f = s; xs = [t1;ti;tj] ; ty = Ty.Tbitv n} 
+      let rec make_rec t' ctx = match T.view t' with
+	| {T.f = Sy.Bitv s } -> string_to_bitv s, ctx
+	| {T.f = Sy.Binop Sy.At ; xs = [t1;t2] ; ty = Ty.Tbitv n} ->
+	    let r1, ctx = make_rec t1 ctx in
+	    let r2, ctx = make_rec t2 ctx in
+	    { bv = I_Comp (r1, r2) ; sz = n }, ctx
+	| {T.f = s; xs = [t1;ti;tj] ; ty = Ty.Tbitv n} 
 	    when Sy.equal s Sy.shat ->
 	    begin 
-	      match Term.view ti , Term.view tj with
-		| { Term.f = Sy.Int i } , { Term.f = Sy.Int j } -> 
+	      match T.view ti , T.view tj with
+		| { T.f = Sy.Int i } , { T.f = Sy.Int j } -> 
 		    let i = int_of_string i.node in
 		    let j = int_of_string j.node in
-		    { sz = j - i + 1 ; bv = I_Ext (make_rec t1,i,j)} 
+		    let r1, ctx = make_rec t1 ctx in
+		    { sz = j - i + 1 ; bv = I_Ext (r1,i,j)}, ctx
 		| _ -> assert false
 	    end
-	| {Term.ty = Ty.Tbitv n} -> 
-	    {bv = I_Other (Term (X.make t')) ; sz = n}
+	| {T.ty = Ty.Tbitv n} -> 
+	    let r', ctx' = X.make t' in
+	    let ctx = ctx' @ ctx in
+	    {bv = I_Other (Alien r') ; sz = n}, ctx
 	| _ -> assert false
-      in sigma (make_rec t)   
+      in 
+      let r, ctx = make_rec t [] in
+      sigma r, ctx
   end
 
   module Print = struct
@@ -233,16 +258,16 @@ module Make(X : ALIEN) = struct
       
     let rec print_I_ast fmt ast = match ast.bv with
       | I_Cte b -> fprintf fmt "%d[%d]@?" (if b then 1 else 0) ast.sz
-      | I_Other (Term t) -> fprintf fmt "%a[%d]@?" X.print t ast.sz
+      | I_Other (Alien t) -> fprintf fmt "%a[%d]@?" X.print t ast.sz
       | I_Other (Var tv) -> fprintf fmt "%a@?" print_tvar (tv,ast.sz)
       | I_Ext (u,i,j) -> fprintf fmt "%a<%d,%d>@?" print_I_ast u i j
       | I_Comp(u,v) -> fprintf fmt "@[(%a * %a)@]" print_I_ast u print_I_ast v
    
     let print fmt ast = match ast.bv with 
       | Cte b -> fprintf fmt "%d[%d]@?" (if b then 1 else 0) ast.sz
-      | Other (Term t) -> fprintf fmt "%a@?" X.print t
+      | Other (Alien t) -> fprintf fmt "%a@?" X.print t
       | Other (Var tv) -> fprintf fmt "%a@?" print_tvar (tv,ast.sz)   
-      | Ext (Term t,sz,i,j) -> 
+      | Ext (Alien t,sz,i,j) -> 
 	  fprintf fmt "%a@?" X.print t;
 	  fprintf fmt "<%d,%d>@?" i j
       | Ext (Var tv,sz,i,j) -> 
@@ -282,7 +307,9 @@ module Make(X : ALIEN) = struct
       fprintf fmt "\nSolution :\n";
       List.iter
 	(fun (a,value) -> 
-	   fprintf fmt " %a = %a \n" print a print_C_ast value ) l
+	   fprintf fmt " %a = %a \n" print a print_C_ast value ) l;
+      fprintf fmt "@."
+      
   end 
 
   module Solver = struct
@@ -395,8 +422,8 @@ module Make(X : ALIEN) = struct
 	|Cte b, Other (Var _) -> [cte_vs_other b st2]
 	|Other (Var _), Cte b -> [cte_vs_other b st1]
 
-	|Cte b, Other (Term t) -> [cte_vs_other b st2]
-	|Other (Term t), Cte b -> [cte_vs_other b st1]
+	|Cte b, Other (Alien t) -> [cte_vs_other b st2]
+	|Other (Alien t), Cte b -> [cte_vs_other b st1]
 
 	|Cte b, Ext(xt,s_xt,i,j) -> [cte_vs_ext b xt s_xt i j]
 	|Ext(xt,s_xt,i,j), Cte b -> [cte_vs_ext b xt s_xt i j]
@@ -420,16 +447,7 @@ module Make(X : ALIEN) = struct
 	  else (t',cnf')::(add r (t,cnf))
       in List.fold_left add [] l
 
-(*
-    let partition liste = 
-      let rec fold2 (t,ct,c) = function
-	|[] -> ((t,ct),c)
-	|(x,l)::r -> fold2 (if x=t then (t,l::ct,c) else (t,ct,(x,l)::c)) r
-      in let rec f_rec acc = function
-	|[] -> acc
-	|(t,l)::r -> let (c1,c2) = fold2 (t,[l],[]) r in f_rec (c1::acc) c2
-      in f_rec [] liste
-*)
+
     let rec slicing_pattern s_l =
       let rec f_aux l1 l2 = match (l1,l2) with
 	|[],[] -> []
@@ -602,26 +620,28 @@ module Make(X : ALIEN) = struct
 	  if c<>0 then c else comp l1 l2
     in comp b1 b2
     
-  let leaves r = 
-    match X.extract r with
-	None -> assert false
-      | Some bitv ->
-	  List.fold_left 
-	    (fun acc x -> 
-	       match x.bv with 
-		 | Cte _  -> acc
-		 | Ext( Var v,sz,_,_) -> 
-		     (X.embed [{bv=Other (Var v) ; sz = sz }])::acc
-		 | Other (Var _)  -> (X.embed [x])::acc
-		 | Other (Term t) | Ext(Term t,_,_,_) -> t::acc) [] bitv
+  let leaves bitv = 
+    List.fold_left 
+      (fun acc x -> 
+	 match x.bv with 
+	   | Cte _  -> acc
+	   | Ext( Var v,sz,_,_) -> 
+	       (X.embed [{bv=Other (Var v) ; sz = sz }])::acc
+	   | Other (Var _)  -> (X.embed [x])::acc
+	   | Other (Alien t) | Ext(Alien t,_,_,_) -> (X.leaves t)@acc
+      ) [] bitv
 
-  let is_mine = function [{bv = Other (Term r)}] -> r | bv -> X.embed bv
+  let is_mine = function [{bv = Other (Alien r)}] -> r | bv -> X.embed bv
 
   let print = Print.print_C_ast
 
-  let make t = is_mine (Canonizer.make t)
+  let make t = 
+    let r, ctx = Canonizer.make t in
+    is_mine r, ctx
 
-  let type_infos bv = 
+  let color _ = assert false
+
+  let type_info bv = 
     let sz = List.fold_left (fun acc bv -> bv.sz + acc) 0 bv in
     Ty.Tbitv sz
   
@@ -642,48 +662,53 @@ module Make(X : ALIEN) = struct
       ) (f_aux (List.hd biv)) (List.tl biv)
       
   let size_of r = 
-    match X.type_infos r with Ty.Tbitv i -> i | _ -> 
+    match X.type_info r with Ty.Tbitv i -> i | _ -> 
       Format.eprintf "ici=%a@." X.print r;
       assert false
     
   let extract r ty = 
     match X.extract r with 
 	Some (u::_ as bv) -> to_i_ast bv
-      | None -> {bv =  Canonizer.I_Other (Term r); sz = ty}
+      | None -> {bv =  Canonizer.I_Other (Alien r); sz = ty}
       | Some [] -> assert false
 
   let extract_xterm r = 
     match X.extract r with 
 	Some ([{bv=Other(Var _ as x)}]) -> x
-      | None -> Term r
+      | None -> Alien r
       | _ -> assert false
 
   let var_or_term x = 
     match x.bv with
 	Other (Var _) -> X.embed [x]
-      | Other (Term r) -> r
+      | Other (Alien r) -> r
       | _ -> assert false
 
-  let solve u t = 
-    if Options.debug_bitv then eprintf "[Bitv] %a = %a@." X.print u X.print t;
-    let u , t =    
-      match X.extract u , X.extract t with 
-	  Some u , Some t -> u,t 
-	| None , Some t -> [{bv=Other(Term u); sz=size_of u}] , t
-	| Some u , None -> u , [{bv=Other(Term t); sz=size_of t}]
-	| _ -> assert false
-    in
-    try
-      List.map (fun (p,v) -> var_or_term p,is_mine v) (Solver.solve u t)
-    with Solver.Valid -> []
+
+  (* ne resout pas quand c'est deja resolu *)
+  let solve repr u t = 
+    if Options.debug_bitv then 
+      eprintf "[Bitv] solve %a = %a@." X.print u X.print t;
     
+    match X.extract u , X.extract t with 
+      | None   , None   -> assert false
+      | None   , Some _ -> [u , t]
+      | Some _ , None   -> [t , u]
+      | Some u , Some t -> 
+	  try
+	    List.map 
+	      (fun (p,v) -> var_or_term p,is_mine v) 
+	      (Solver.solve u t)
+	  with Solver.Valid -> []
+	      
+
  
   let rec subst_rec x subs biv = 
     match biv.bv , extract_xterm x with
       | Canonizer.I_Cte _ , _ -> biv
       | Canonizer.I_Other (Var y) , Var z when y=z -> extract subs biv.sz
       | Canonizer.I_Other (Var _) , _ -> biv
-      | Canonizer.I_Other (Term tt) , _ -> 
+      | Canonizer.I_Other (Alien tt) , _ -> 
 	  if X.compare x tt = 0 then 
 	    extract subs biv.sz
 	  else extract (X.subst x subs tt) biv.sz
@@ -701,6 +726,13 @@ module Make(X : ALIEN) = struct
       let r = Canonizer.sigma (subst_rec x subs (to_i_ast biv)) in
       is_mine r
 
+    module M =  Map.Make 
+      (struct 
+         type t = X.r
+         let compare = X.compare 
+       end)
+
+
   module Map = Map.Make 
     (struct 
        type t = (X.r simple_term) list
@@ -713,14 +745,70 @@ module Make(X : ALIEN) = struct
       let compare = compare
     end)
 
-  module R =
-  struct
+  module Rel = struct
+
+    type r = X.r
+        (* ces X.t sont toujours des constantes *)
+        (* Cette map associe à chaque représentant r de type bitvector
+           une constante c tel que toutes les constantes plus petite
+           que c sont différentes de r.
+           Un représentant r est associé à None si c'est une constante*)
     type elt = t
-    type t = unit
+    type t =  (elt option) M.t
+
     exception Inconsistent    
-    let empty _ = ()
-    let assume _ _ _ _ = ((),Literal.SetEq.empty)
-    let query _ _ _ _ = assert false
+    let empty () = M.empty
+
+    let is_bitv t = 
+      match (Term.view t).T.ty with 
+        | Ty.Tbitv _ -> true
+        | _ -> false
+
+    let assume env la = 
+      let leqs = 
+	List.fold_left 
+          (fun acc (a, root) -> 
+             match a with
+               | Literal.Neq(rx1,rx2) ->
+                   begin
+		     match X.extract rx1, X.extract rx2 with
+                       | Some [r1], Some [r2] when r1.sz = 1 -> 
+                           begin
+			     match r1.bv, r2.bv with
+                               | Cte b, Other o -> 
+				   let v = X.embed [r1] in
+				   (Literal.Eq(rx2,v),root)::acc  
+			       | Other o, Cte b -> 
+				   let v = X.embed [r2] in
+				   (Literal.Eq(rx1,v),root)::acc  
+                               | _ -> acc
+			   end
+                       | _ -> acc
+                   end
+               | _ -> acc) [] la
+      in 
+      env, leqs
+ 
+    let cst0 n = 
+      let rec aux acc = function
+        | n when n <= 0 -> acc
+        | n -> aux ({bv=Cte false;sz=1}::acc) (n-1) in
+      aux [] n
+        
+    let add env r = 
+      match X.type_info r with
+        | Ty.Tbitv i when not (M.mem r env) ->     
+            let cst0 = Some (cst0 i) in
+            M.add r cst0 env
+        | _ -> env
+
+    let rec succ = function
+      | [] -> raise Exception.Inconsistent
+      | ({bv=false} as x)::l -> {x with bv = true }::l
+      | ({bv=true } as x)::l -> {x with bv = false}::(succ l)
+
+    let case_split env = env, []    
+    let query _ _ = false
   end
 
 end
