@@ -1,11 +1,15 @@
 (**************************************************************************)
 (*                                                                        *)
 (*     The Alt-ergo theorem prover                                        *)
-(*     Copyright (C) 2006-2008                                            *)
+(*     Copyright (C) 2006-2010                                            *)
 (*                                                                        *)
 (*     Sylvain Conchon                                                    *)
 (*     Evelyne Contejean                                                  *)
-(*     CNRS-LRI-Universite Paris Sud                                      *)
+(*     Stephane Lescuyer                                                  *)
+(*     Mohamed Iguernelala                                                *)
+(*     Alain Mebsout                                                      *)
+(*                                                                        *)
+(*     CNRS - INRIA - Universite Paris Sud                                *)
 (*                                                                        *)
 (*   This file is distributed under the terms of the CeCILL-C licence     *)
 (*                                                                        *)
@@ -23,7 +27,7 @@ let rec find_eq x eqs = function
       if eq_var x t1 then (x,t2)::eqs 
       else if eq_var x t2 then (x,t1)::eqs
       else eqs
-  | TFop(_,l) -> List.fold_left (find_eq x) eqs l
+  | TFop(OPand,l) -> List.fold_left (find_eq x) eqs l
   | _ -> eqs (* XXX: TODO *)
 
 let find_equalities lv f = 
@@ -66,5 +70,8 @@ let make_instance f =
   apply_subst_formula (List.map List.hd lt) f.qf_form
 
 let make f = 
-  try TFop(OPor,[TFexists f;make_instance f]) with Not_found -> TFexists f
+  if Options.no_rm_eq_existential 
+  then TFexists f
+  else
+    try (*TFop(OPor,[TFexists f;*)make_instance f(*])*) with Not_found -> TFexists f
 
