@@ -1,11 +1,15 @@
 (**************************************************************************)
 (*                                                                        *)
 (*     The Alt-ergo theorem prover                                        *)
-(*     Copyright (C) 2006-2008                                            *)
+(*     Copyright (C) 2006-2010                                            *)
 (*                                                                        *)
 (*     Sylvain Conchon                                                    *)
 (*     Evelyne Contejean                                                  *)
-(*     CNRS-LRI-Universite Paris Sud                                      *)
+(*     Stephane Lescuyer                                                  *)
+(*     Mohamed Iguernelala                                                *)
+(*     Alain Mebsout                                                      *)
+(*                                                                        *)
+(*     CNRS - INRIA - Universite Paris Sud                                *)
 (*                                                                        *)
 (*   This file is distributed under the terms of the CeCILL-C licence     *)
 (*                                                                        *)
@@ -13,6 +17,19 @@
 
 type gsubst = { sbt : Term.subst ; gen : int ; goal : bool}
 
+type pat_info = {
+  pat_age : int ; 
+  pat_orig : Formula.t ; 
+  pat_formula : Formula.t ; 
+  pat_dep : Explanation.t ;
+}
+
+type term_info = {
+  term_age : int ; 
+  term_from_goal : bool ;
+  term_orig : Formula.t option
+}
+
 module type X = sig
   type t
 
@@ -25,8 +42,10 @@ module type S = sig
   type uf
 
   val empty : t
-  val add : int -> bool -> Formula.t option -> Term.t -> t -> t 
-  val matching : Formula.t -> Term.t list -> t -> uf ->  gsubst list
+  val add_term : term_info -> Term.t -> t -> t 
+  val add_pat : pat_info * Term.t list -> t -> uf -> t
+  val query : t -> uf -> (pat_info * gsubst list) list
+
 end
 
 module Make (X : X) : S with type uf = X.t
