@@ -1,11 +1,15 @@
 (**************************************************************************)
 (*                                                                        *)
 (*     The Alt-ergo theorem prover                                        *)
-(*     Copyright (C) 2006-2008                                            *)
+(*     Copyright (C) 2006-2010                                            *)
 (*                                                                        *)
 (*     Sylvain Conchon                                                    *)
 (*     Evelyne Contejean                                                  *)
-(*     CNRS-LRI-Universite Paris Sud                                      *)
+(*     Stephane Lescuyer                                                  *)
+(*     Mohamed Iguernelala                                                *)
+(*     Alain Mebsout                                                      *)
+(*                                                                        *)
+(*     CNRS - INRIA - Universite Paris Sud                                *)
 (*                                                                        *)
 (*   This file is distributed under the terms of the CeCILL-C licence     *)
 (*                                                                        *)
@@ -21,23 +25,16 @@ type 'a abstract =
   | Other of 'a * Ty.t
 
 module type ALIEN = sig
-  type r
-  val make : Term.t -> r
-  val type_infos : r -> Ty.t
-  val extract : r -> (r abstract) option
+  include Sig.X
   val embed : r abstract -> r
-  val subst : r -> r -> r -> r
-  val leaves : r -> r list
-  val compare : r -> r -> int
-  val print : Format.formatter -> r -> unit
+  val extract : r -> (r abstract) option
 end
 
-
 module Make (X : ALIEN) = struct 
 
   module XS = Set.Make(struct type t = X.r let compare = X.compare end)
 
-  let name = "pairs "
+  let name = "pairs"
 
   type t = X.r abstract
   type r = X.r
@@ -102,30 +99,48 @@ module Make (X : ALIEN) = struct
       | x -> X.embed x
 
   let make t = 
-    let rec make_rec t =
+    let rec make_rec t ctx =
       match (Term.view t) with
 	| {Term.f = Symbols.Name (f,_); xs = l; ty=ty} ->
 	    begin
 	      if is_pair f then
 		match l with 
-		  | [lc; rc] -> Pair (make_rec lc, make_rec rc,ty)
+		  | [lc; rc] -> 
+		      let f, ctx = make_rec lc ctx in
+		      let s, ctx = make_rec rc ctx in
+		      Pair (f, s,ty), ctx
 		  | _ -> assert false
 	      else if is_fst f then
 		match l with
-		  | [p] -> Fst (make_rec p,ty)
+		  | [p] -> 
+		      let f, ctx = make_rec p ctx in
+		      Fst (f,ty), ctx
 		  | _ -> assert false
 	      else if is_snd f then
 		match l with
-		  | [p] -> Snd (make_rec p,ty)
+		  | [p] -> 
+		      let s, ctx = make_rec p ctx in
+		      Snd (s,ty), ctx
 		  | _ -> assert false
 	      else
-		Other (X.make t,ty)
+		let r, ctx' = X.make t in
+		let ctx = ctx'@ctx in
+		Other (r,ty), ctx
 	    end
-	| {Term.ty=ty} -> Other (X.make t,ty)
+	| {Term.ty=ty} -> 
+	    let r, ctx' = X.make t in
+	    let ctx = ctx'@ctx in
+	    Other (r,ty), ctx
     in 
-    is_mine (make_rec t)
+    let r, ctx = make_rec t [] in
+    is_mine r, ctx
 
-  let embedding r = Other(r,X.type_infos r)
+  let color _ = assert false
+    
+  let embed r = 
+    match X.extract r with
+      | Some p -> p
+      | None -> Other(r, X.type_info r)
 
   let xs_of_list = List.fold_left (fun s x -> XS.add x s) XS.empty
 
@@ -137,9 +152,7 @@ module Make (X : ALIEN) = struct
 	| Snd (p,_) -> leaves p
 	| Other(x,_) -> xs_of_list (X.leaves x)
     in
-    match X.extract t with
-	None -> assert false
-      | Some t -> XS.elements (leaves t)
+    XS.elements (leaves t)
 
   let compare t u =
     raw_comp (normalize t) (normalize u)
@@ -246,10 +259,9 @@ module Make (X : ALIEN) = struct
 	  
   let subst p v r = is_mine (subst_rec p v r)
       
-  let is_mine_hs t = 
-    match (Term.view t) with
-      | {Term.f = Symbols.Name (f,_)} -> is_fst f || is_snd f || is_pair f
-      | _ -> false
+  let is_mine_symb =  function 
+    | Symbols.Name (f,_) when pairs -> is_fst f || is_snd f || is_pair f
+    | _ -> false
       
   let is_mine_a _ = false
 
@@ -258,17 +270,17 @@ module Make (X : ALIEN) = struct
     | Other _ -> false
     | Fst(t,_) | Snd(t,_) -> is_mine_type t
 	
-  let normal_form a = a
+  let existential _ = false
 
-  let type_infos = 
+  let type_info = 
     function Pair(_,_,t) | Fst(_,t) | Snd(_,t) | Other(_,t) -> t
 
   let type_snd t = 
-    match type_infos t with 
+    match type_info t with 
 	Ty.Text(([_;ty] | [ty]),_) -> ty | _ -> assert false
 
   let type_fst t = 
-    match type_infos t with Ty.Text(([ty;_]|[ty]),_) -> ty  
+    match type_info t with Ty.Text(([ty;_]|[ty]),_) -> ty  
       | _ -> assert false
 
 (** [decompose uf t] adds [t] and all its subterms to [uf] *)
@@ -279,12 +291,12 @@ module Make (X : ALIEN) = struct
 	add p (dec (dec uf t) u)
     | Fst(t,ty) as p ->
 	let t' = Snd(t,type_snd t) in
-	let t'' = Pair(p,t',type_infos t) in
+	let t'' = Pair(p,t',type_info t) in
 	let uf = add t' (add t'' (add p (dec uf t))) in
 	  UF.union uf t'' t
     | Snd(t,_) as p ->
 	let t' = Fst(t,type_fst t) in
-	let t'' = Pair(t',p,type_infos t) in
+	let t'' = Pair(t',p,type_info t) in
 	let uf = add t' (add t'' (add p (dec uf t))) in
 	  UF.union uf t'' t
     | p -> add p uf
@@ -352,7 +364,7 @@ module Make (X : ALIEN) = struct
       (** we search all kinds of equations *)
       find_fst_pair uf (find_snd_pair uf (find_pair_pair uf []))
 	
-  let solve r1 r2 = 
+  let solve repr r1 r2 = 
     let leaves t = 
       let rec leaves t = 
 	match (normalize t) with
@@ -367,9 +379,9 @@ module Make (X : ALIEN) = struct
       printf "[pairs] Solving %a = %a @." X.print r1 X.print r2;
     (* populate the uf *)
     let t1 = match X.extract r1 with 
-	Some t1 -> t1 | None -> Other(r1,X.type_infos r1) in
+	Some t1 -> t1 | None -> Other(r1, X.type_info r1) in
     let t2 = match X.extract r2 with 
-	Some t2 -> t2 | None -> Other(r2,X.type_infos r2) in
+	Some t2 -> t2 | None -> Other(r2, X.type_info r2) in
     let uf = decompose UF.Right (decompose UF.Left UF.empty t1) t2 in
     (* merge t1 and t2 *)
     let uf' = UF.union uf t1 t2 in
@@ -388,7 +400,7 @@ module Make (X : ALIEN) = struct
     let f tag =
       List.fold_left
 	(fun acc t -> 
-	   match UF.find_new_mate ufsat (Other(t,X.type_infos t)) tag with
+	   match UF.find_new_mate ufsat (Other(t, X.type_info t)) tag with
 	     | Some e -> (t, is_mine e)::acc
 	     | None -> (* Format.printf "ici : %a\n" X.print t; *) 
 		 acc) []
@@ -407,20 +419,28 @@ module Make (X : ALIEN) = struct
 (* 	    else  *)
 (* 	      sols (l2,UF.Left) (l1,UF.Right) *)
     in
+
     if debug_pairs then 
       (printf "[pairs] Results :@ ";
        List.iter (fun (t, r) -> 
-		    printf "%a = %a@ ;@ " X.print t X.print r) res);
-    res
+		    printf "%a |-> %a@ ;@ " X.print t X.print r) res);
+    
+    (* AC ordering : Take 1*) 
+    List.fold_left
+      (fun acc (p,v) ->
+	 if X.is_empty v && X.compare  v p > 0 then (v,p) :: acc 
+	 else (p,v) :: acc
+      )[] res
           
-  module R =
+  module Rel =
   struct
-    type elt = t
+    type r = X.r
     type t = unit
     exception Inconsistent    
     let empty _ = ()
-    let assume _ _ _ _ = ((),Literal.SetEq.empty)
-    let query _ _ _ _ = assert false
+    let assume _ _ = (), []
+    let query _ _  = false
+    let case_split env = env, []
+    let add env _ = env
   end
-
 end
