@@ -1,11 +1,15 @@
 (**************************************************************************)
 (*                                                                        *)
 (*     The Alt-ergo theorem prover                                        *)
-(*     Copyright (C) 2006-2008                                            *)
+(*     Copyright (C) 2006-2010                                            *)
 (*                                                                        *)
 (*     Sylvain Conchon                                                    *)
 (*     Evelyne Contejean                                                  *)
-(*     CNRS-LRI-Universite Paris Sud                                      *)
+(*     Stephane Lescuyer                                                  *)
+(*     Mohamed Iguernelala                                                *)
+(*     Alain Mebsout                                                      *)
+(*                                                                        *)
+(*     CNRS - INRIA - Universite Paris Sud                                *)
 (*                                                                        *)
 (*   This file is distributed under the terms of the CeCILL-C licence     *)
 (*                                                                        *)
@@ -14,15 +18,9 @@
 type 'a abstract
 
 module type ALIEN = sig
-  type r
-  val make : Term.t -> r
-  val type_infos : r -> Ty.t
-  val extract : r -> (r abstract) option
+  include Sig.X
   val embed : r abstract -> r
-  val subst : r -> r -> r -> r
-  val leaves : r -> r list
-  val compare : r -> r -> int
-  val print : Format.formatter -> r -> unit
+  val extract : r -> (r abstract) option
 end
 
 module Make 
