@@ -1,11 +1,15 @@
 (**************************************************************************)
 (*                                                                        *)
 (*     The Alt-ergo theorem prover                                        *)
-(*     Copyright (C) 2006-2008                                            *)
+(*     Copyright (C) 2006-2010                                            *)
 (*                                                                        *)
 (*     Sylvain Conchon                                                    *)
 (*     Evelyne Contejean                                                  *)
-(*     CNRS-LRI-Universite Paris Sud                                      *)
+(*     Stephane Lescuyer                                                  *)
+(*     Mohamed Iguernelala                                                *)
+(*     Alain Mebsout                                                      *)
+(*                                                                        *)
+(*     CNRS - INRIA - Universite Paris Sud                                *)
 (*                                                                        *)
 (*   This file is distributed under the terms of the CeCILL-C licence     *)
 (*                                                                        *)
@@ -227,16 +231,16 @@ and envformula env f = pexp f.floc (form_to_desc env f.formula)
 let formula  = envformula {uterm = StringMap.empty ; uformula = StringMap.empty}
 
 let psig_to_pdef (pos,psig) =
-  Logic (pos,false, false,[psig.pname],
+  Logic (pos,Symbols.No,[psig.pname],
 	 PPredicate (List.map curry_sort_to_type psig.pargs))
 
 let fsig_to_fdef (pos,fsig) =
-  Logic (pos,false, false,[fsig.fname],
+  Logic (pos,Symbols.No,[fsig.fname],
 	 PFunction ((List.map curry_sort_to_type fsig.fargs),
                     curry_sort_to_type fsig.fres ))
 
 let sortdecl_to_type (pos,s) =
-  TypeDecl (pos,false,[], s)
+  TypeDecl (pos,[], s)
     
 
 let bench_to_why = function
