@@ -1,11 +1,15 @@
 (**************************************************************************)
 (*                                                                        *)
 (*     The Alt-ergo theorem prover                                        *)
-(*     Copyright (C) 2006-2008                                            *)
+(*     Copyright (C) 2006-2010                                            *)
 (*                                                                        *)
 (*     Sylvain Conchon                                                    *)
 (*     Evelyne Contejean                                                  *)
-(*     CNRS-LRI-Universite Paris Sud                                      *)
+(*     Stephane Lescuyer                                                  *)
+(*     Mohamed Iguernelala                                                *)
+(*     Alain Mebsout                                                      *)
+(*                                                                        *)
+(*     CNRS - INRIA - Universite Paris Sud                                *)
 (*                                                                        *)
 (*   This file is distributed under the terms of the CeCILL-C licence     *)
 (*                                                                        *)
@@ -14,7 +18,9 @@
 open Hashcons
 
 type binop = Plus | Minus | Mult | Div | Modulo | At
-type is_ac = bool
+
+type is_ac = No | Ac
+
 type t = 
     Name of Hstring.t * is_ac
   | Int of Hstring.t
@@ -23,16 +29,16 @@ type t =
   | Binop of binop 
   | Var of Hstring.t
 
-let unit = Name (Hstring.make "()", false)
+let unit = Name (Hstring.make "()", No)
 
-let name ?(ac=false) s = Name (Hstring.make s, ac)
+let name ?(ac=No) s = Name (Hstring.make s, ac)
 
 let var s = Var (Hstring.make s)
 let int i = Int (Hstring.make i)
 let rat r = Rat (Hstring.make r)
 
 let is_ac = function
-    Name(_,b) -> b
+    Name(_,Ac) -> true
   | _ -> false
 
 let underscoring = function
@@ -65,13 +71,16 @@ let compare s1 s2 =  match s1,s2 with
 let equal s1 s2 = compare s1 s2 = 0
 
 let hash = function
-    Name (n,ac) -> Hstring.hash n * 19 + (if ac then 1 else 0)
+  | Name (n,Ac) -> Hstring.hash n * 19 + 1
+  | Name (n,No) -> Hstring.hash n * 19
+
+
   | Var n (*| Int n*) -> Hstring.hash n * 19 + 1
   | s -> Hashtbl.hash s
 	
 let to_string =  function
     Name (n,_) -> Hstring.view n
-  | Var x -> "'"^(Hstring.view x)^"'"
+  | Var x -> (Hstring.view x)
   | Int n -> Hstring.view n
   | Rat n -> Hstring.view n
   | Bitv s -> "["^s^"]"
@@ -84,12 +93,22 @@ let to_string =  function
 
 let print fmt s = Format.fprintf fmt "%s" (to_string s)
 
-let dummy = Name (Hstring.make "_one", false)
+let dummy = Name (Hstring.make "_one", No)
 
 let fresh = 
   let cpt = ref 0 in
   fun s -> incr cpt; name (Format.sprintf "__%s%i" s (!cpt))
 
+
+let f_get = Hstring.make "get"
+let f_set = Hstring.make "set"
+
+let is_get = function 
+    Name(hs,_) ->  Hstring.compare hs f_get = 0 | _ -> false 
+      
+let is_set = function 
+    Name(hs,_) ->  Hstring.compare hs f_set = 0 | _ -> false 
+
 module Map =
   Map.Make(struct type t' = t type t=t' let compare=compare end)
 
