@@ -1,18 +1,24 @@
 (**************************************************************************)
 (*                                                                        *)
 (*     The Alt-ergo theorem prover                                        *)
-(*     Copyright (C) 2006-2008                                            *)
+(*     Copyright (C) 2006-2010                                            *)
 (*                                                                        *)
 (*     Sylvain Conchon                                                    *)
 (*     Evelyne Contejean                                                  *)
-(*     CNRS-LRI-Universite Paris Sud                                      *)
+(*     Stephane Lescuyer                                                  *)
+(*     Mohamed Iguernelala                                                *)
+(*     Alain Mebsout                                                      *)
+(*                                                                        *)
+(*     CNRS - INRIA - Universite Paris Sud                                *)
 (*                                                                        *)
 (*   This file is distributed under the terms of the CeCILL-C licence     *)
 (*                                                                        *)
 (**************************************************************************)
 
 type binop = Plus | Minus | Mult | Div | Modulo | At
-type is_ac = bool
+
+type is_ac = No | Ac
+
 type t = 
     Name of Hstring.t * is_ac
   | Int of Hstring.t
@@ -43,10 +49,14 @@ val equal : t -> t -> bool
 val compare : t -> t -> int
 val hash : t -> int
 
+val to_string : t -> string
 val print : Format.formatter -> t -> unit
 
 val dummy : t
 val fresh : string -> t
+
+val is_get : t -> bool 
+val is_set : t -> bool 
   
 module Map : Map.S with type key = t
 module Set : Set.S with type elt = t
