@@ -1,11 +1,15 @@
 (**************************************************************************)
 (*                                                                        *)
 (*     The Alt-ergo theorem prover                                        *)
-(*     Copyright (C) 2006-2008                                            *)
+(*     Copyright (C) 2006-2010                                            *)
 (*                                                                        *)
 (*     Sylvain Conchon                                                    *)
 (*     Evelyne Contejean                                                  *)
-(*     CNRS-LRI-Universite Paris Sud                                      *)
+(*     Stephane Lescuyer                                                  *)
+(*     Mohamed Iguernelala                                                *)
+(*     Alain Mebsout                                                      *)
+(*                                                                        *)
+(*     CNRS - INRIA - Universite Paris Sud                                *)
 (*                                                                        *)
 (*   This file is distributed under the terms of the CeCILL-C licence     *)
 (*                                                                        *)
@@ -40,23 +44,15 @@ let tbl = T.create 100007
   
 let view t = t.node
 
-let compare t1 t2 = Pervasives.compare t1.tag t2.tag
+
+(* fresh variables must be smaller than problem's variables.
+   thus, Instead of comparinf t1.tag with t2.tag, 
+   we compare t2.tag and t1.tag *)
+let compare t1 t2 = Pervasives.compare t2.tag t1.tag
 
 let sort = List.sort compare
 
-let make s l ty = 
-  if not (Symbols.is_ac s) then 
-    T.hashcons tbl {f=s;xs=l;ty=ty}
-  else
-    begin
-      let l'  = 
-	List.fold_left 
-	  (fun acc t -> 
-	     let {f=s'; xs=xs';ty=ty'} = view t in 
-	     if Symbols.equal s s' then xs'@acc else t::acc) [] l 
-      in
-      T.hashcons tbl {f=s; xs=sort l';ty=ty}
-    end
+let make s l ty = T.hashcons tbl {f=s;xs=l;ty=ty}
 
 let shorten t = 
   let {f=f;xs=xs;ty=ty} = view t in
@@ -72,8 +68,19 @@ let bitv bt ty = make (Sy.Bitv bt) [] ty
 
 let rec print fmt t = 
   let {f=x;xs=l;ty=ty} = view t in
-  if l=[] then Format.fprintf fmt "%a" Sy.print x (*Ty.print ty*)
-  else Format.fprintf fmt "%a(%a)" Sy.print x print_list l
+  match l, Options.debug_types with
+    | [], true  ->
+        Format.fprintf fmt "%a [%a]" Sy.print x Ty.print ty
+          
+    | [], false -> 
+        Format.fprintf fmt "%a" Sy.print x
+          
+    | _ , true  -> 
+        Format.fprintf fmt "%a(%a) [%a]" Sy.print x print_list l Ty.print ty
+          
+    | _ , false ->
+        Format.fprintf fmt "%a(%a)" Sy.print x print_list l
+
 and print_list fmt = function
   | [] -> ()
   | [t] -> print fmt t
