@@ -1,11 +1,15 @@
 (**************************************************************************)
 (*                                                                        *)
 (*     The Alt-ergo theorem prover                                        *)
-(*     Copyright (C) 2006-2008                                            *)
+(*     Copyright (C) 2006-2010                                            *)
 (*                                                                        *)
 (*     Sylvain Conchon                                                    *)
 (*     Evelyne Contejean                                                  *)
-(*     CNRS-LRI-Universite Paris Sud                                      *)
+(*     Stephane Lescuyer                                                  *)
+(*     Mohamed Iguernelala                                                *)
+(*     Alain Mebsout                                                      *)
+(*                                                                        *)
+(*     CNRS - INRIA - Universite Paris Sud                                *)
 (*                                                                        *)
 (*   This file is distributed under the terms of the CeCILL-C licence     *)
 (*                                                                        *)
@@ -17,12 +21,17 @@ module type S = sig
   module R : Sig.X
 
   val empty :  t
-  val add : t -> Term.t -> t
+  val add : t -> Term.t -> t * Literal.t list
+
+  val add_semantic : t -> R.r -> t 
+
   val mem : t -> Term.t -> bool
+
   val find : t -> Term.t -> R.r
+  val find_r : t -> R.r -> R.r
+
   val union : 
-    t -> Term.t -> Term.t -> Explanation.t ->
-    t * (R.r * ((R.r * R.r) list) * R.r) list
+    t -> R.r -> R.r -> Explanation.t -> t * (R.r * (R.r * R.r) list * R.r) list
   
   val distinct : t -> Term.t -> Term.t -> Explanation.t -> t
 
@@ -32,7 +41,11 @@ module type S = sig
     
   val explain : t -> Term.t -> Term.t -> Explanation.t
   val neq_explain : t -> Term.t -> Term.t -> Explanation.t
+  
+  val print : Format.formatter -> t -> unit
 
+  val distinct_r : 
+    t -> R.r -> R.r -> Explanation.t -> Explanation.t -> Explanation.t -> t
 end
 
 module Make ( X : Sig.X ) : S with module R = X
