@@ -1,11 +1,15 @@
 (**************************************************************************)
 (*                                                                        *)
 (*     The Alt-ergo theorem prover                                        *)
-(*     Copyright (C) 2006-2008                                            *)
+(*     Copyright (C) 2006-2010                                            *)
 (*                                                                        *)
 (*     Sylvain Conchon                                                    *)
 (*     Evelyne Contejean                                                  *)
-(*     CNRS-LRI-Universite Paris Sud                                      *)
+(*     Stephane Lescuyer                                                  *)
+(*     Mohamed Iguernelala                                                *)
+(*     Alain Mebsout                                                      *)
+(*                                                                        *)
+(*     CNRS - INRIA - Universite Paris Sud                                *)
 (*                                                                        *)
 (*   This file is distributed under the terms of the CeCILL-C licence     *)
 (*                                                                        *)
@@ -18,7 +22,7 @@ type constant =
   | ConstInt of string
   | ConstReal of string
   | ConstNum of Num.num
-  | ConstUnit
+  | ConstUnit (* utile ? *)
 
 type pp_infix = 
   PPand | PPor | PPimplies | PPiff |
@@ -52,26 +56,25 @@ and pp_desc =
   | PPforall of string list* ppure_type * lexpr list list * lexpr
   | PPexists of string * ppure_type * lexpr
   | PPnamed of string * lexpr
+  (* utilisee dans Why ? *)
   | PPlet of string * lexpr * lexpr
 
 (* Declarations. *)
 
-type external_ = bool
-
 type plogic_type =
   | PPredicate of ppure_type list
   | PFunction of ppure_type list * ppure_type
 
-type is_ac = bool
+type is_ac = Symbols.is_ac
 
 type decl = 
   | Axiom of loc * string * lexpr
   | Goal of loc * string * lexpr
-  | Logic of loc * is_ac * external_ * string list * plogic_type
+  | Logic of loc * is_ac * string list * plogic_type
   | Predicate_def of loc * string * (loc * string * ppure_type) list * lexpr
   | Function_def 
       of loc * string * (loc * string * ppure_type) list * ppure_type * lexpr
-  | TypeDecl of loc * external_ * string list * string
+  | TypeDecl of loc * string list * string
 
 type file = decl list
 
@@ -127,9 +130,22 @@ and tform =
 type typed_decl = 
   | TAxiom of loc * string * tform
   | TGoal of loc * string * tform
-  | TLogic of loc * external_ * string list * plogic_type
+  | TLogic of loc * string list * plogic_type
   | TPredicate_def of loc * string * (string * ppure_type) list * tform
   | TFunction_def 
       of loc * string * (string * ppure_type) list * ppure_type * tform
-  | TTypeDecl of loc * external_ * string list * string
+  | TTypeDecl of loc * string list * string
   | TWarning of loc
+
+
+(* Sat entry *)
+
+type sat_decl_aux = 
+  | Assume of Formula.t * bool 
+  | PredDef of Formula.t
+  | Query of string * Formula.t * Literal.t list
+
+type sat_tdecl = {
+  st_loc : loc;
+  st_decl : sat_decl_aux
+}
