@@ -1,11 +1,15 @@
 (**************************************************************************)
 (*                                                                        *)
 (*     The Alt-ergo theorem prover                                        *)
-(*     Copyright (C) 2006-2008                                            *)
+(*     Copyright (C) 2006-2010                                            *)
 (*                                                                        *)
 (*     Sylvain Conchon                                                    *)
 (*     Evelyne Contejean                                                  *)
-(*     CNRS-LRI-Universite Paris Sud                                      *)
+(*     Stephane Lescuyer                                                  *)
+(*     Mohamed Iguernelala                                                *)
+(*     Alain Mebsout                                                      *)
+(*                                                                        *)
+(*     CNRS - INRIA - Universite Paris Sud                                *)
 (*                                                                        *)
 (*   This file is distributed under the terms of the CeCILL-C licence     *)
 (*                                                                        *)
@@ -49,11 +53,6 @@ module STRS = Set.Make(
     let compare (t1,_,_) (t2,_,_) = compare_term t1 t2
   end)
 
-type tdecl = 
-    Assume of F.t * loc * bool 
-  | PredDef of F.t * loc
-  | Query of string * F.t * Literal.t list * loc
-
 type error = 
   | BitvExtract of int*int
   | BitvExtractRange of int*int
@@ -261,6 +260,7 @@ module Decl = struct
 		lit , lit::acc
 	    | _ -> assert false
 	  in F.mk_lit a , lit
+
       | TFop((OPand | OPor) as op,[f1;f2]) -> 
 	  let ff1 , lit1 = make_form acc f1 in
 	  let ff2 , lit2 = make_form lit1 f2 in
@@ -287,29 +287,29 @@ module Decl = struct
 	  let upvars = varset_of_list qf.qf_upvars in
 	  let trs = List.map (List.map make_term) qf.qf_triggers in
 	  let ff , lit = make_form acc qf.qf_form in
-	  let fh = 
-	    match f with
-	      | TFforall _ -> F.mk_forall upvars bvars trs ff name
-	      | TFexists _ -> F.mk_exists upvars bvars trs ff name
-	      | _ -> assert false
-	  in fh , lit
-      | TFlet (up,lvar,lterm,lf) -> let ff, lit = make_form acc lf in
-            F.mk_let (varset_of_list up) lvar (make_term lterm) ff, lit
+	  begin match f with
+	    | TFforall _ -> F.mk_forall upvars bvars trs ff name , lit
+	    | TFexists _ -> F.mk_exists upvars bvars trs ff name , lit
+	    | _ -> assert false
+	  end
+      | TFlet (up,lvar,lterm,lf) -> 
+	  let ff, lit = make_form acc lf in
+          F.mk_let (varset_of_list up) lvar (make_term lterm) ff, lit
       | _ -> assert false
     in
     make_form [] f
 
   let push_assume f name loc match_flag = 
     let ff , _ = make_form name f in
-    Queue.push (Assume(ff,loc,match_flag)) queue
+    Queue.push {st_decl=Assume(ff,match_flag) ; st_loc=loc} queue
 
   let push_preddef f name loc match_flag = 
     let ff , _ = make_form name f in
-    Queue.push (PredDef(ff,loc)) queue
+    Queue.push {st_decl=PredDef ff ; st_loc=loc} queue
       
   let push_query n f loc = 
     let ff, lits = make_form "" f in
-    Queue.push (Query(n,ff,lits,loc)) queue
+    Queue.push {st_decl=Query(n,ff,lits) ; st_loc=loc} queue
 
 end
 
@@ -388,14 +388,142 @@ module Triggers = struct
   let sort = 
     List.sort (fun l1 l2 -> compare (List.length l1) (List.length l2) )
       
-  let at_most =
-    let rec atmost acc n l = 
-      match n , l with
-	| 1 , x::_ -> x::acc
-	| _ , x::l -> atmost (x::acc) (n-1) l
-	| _ , [] -> acc
-    in fun n l -> atmost [] n l
+  let compare_tconstant c1 c2 =
+    match c1, c2 with
+      | Tint s1, Tint s2 -> String.compare s1 s2
+      | Tint s1, _ -> 1
+      | _, Tint s1 -> -1
+      | Treal s1, Treal s2 -> String.compare s1 s2
+      | Treal s1, _ -> 1
+      | _, Treal s2 -> -1
+      | Tnum n1, Tnum n2 -> Num.compare_num n1 n2
+      | Tnum n1, _ -> 1
+      | _, Tnum n2 -> -1
+      | Tbitv s1, Tbitv s2 -> String.compare s1 s2
+      | Tbitv s1, _ -> 1
+      | _, Tbitv s2 -> -1
+      | Tunit, Tunit -> 0
 
+  let rec depth_tterm t =
+    match t.tt_desc with
+      | TTtrue -> 0
+      | TTfalse -> 0
+      | TTconst _ -> 0
+      | TTvar _ -> 0
+      | TTapp (_, tl) ->
+	  1 + (List.fold_left
+		 (fun acc t -> max (depth_tterm t) acc) 0 tl)
+      | TTinfix (t1, _, t2) -> 0
+          (* arithmetic triggers are not suitable *)
+          (* max (depth_tterm t1) (depth_tterm t2) *)
+	
+  exception Out of int
+
+  let rec compare_tterm t1 t2 =
+    match t1.tt_desc, t2.tt_desc with
+      | TTtrue, TTtrue -> 0
+      | TTtrue, _ -> -1
+      | _, TTtrue -> 1
+      | TTfalse, TTfalse -> 0
+      | TTfalse, _ -> -1
+      | _, TTfalse -> 1
+      | TTconst c1, TTconst c2 -> compare_tconstant c1 c2
+      | TTconst c1, _ -> -1
+      | _, TTconst c2 -> 1
+      | TTvar v1, TTvar v2 -> Symbols.compare v1 v2
+      | TTvar v1, _ -> -1
+      | _, TTvar v2 -> 1
+      | TTinfix (tu1, s, tu2), TTinfix (tu1', s', tu2') ->
+	  let c = (depth_tterm t1) - (depth_tterm t2) in
+	  if c <> 0 then c
+	  else let c = Symbols.compare s s' in
+	  if c <> 0 then c
+	  else let c = compare_tterm tu1 tu1' in
+	  if c <> 0 then c
+	  else compare_tterm tu2 tu2'
+      | TTinfix (tu1, s, tu2), _ -> -1
+      | _, TTinfix (tu1', s', tu2') -> 1
+      | TTapp (s1, tl1), TTapp (s2, tl2) ->
+	  let l1 = List.map depth_tterm tl1 in
+	  let l2 = List.map depth_tterm tl2 in
+	  let l1 = List.fast_sort compare l1 in
+	  let l2 = List.fast_sort compare l2 in
+	  let c = try
+	    List.iter2
+	      (fun n m -> 
+		 if n <> m then raise (Out (n-m))
+	      ) l1 l2; 0
+	  with 
+	    | Out c -> c 
+	    | _ -> (List.length l1) - (List.length l2) in
+	  if c <> 0 then c
+	  else let c = Symbols.compare s1 s2 in
+	  if c <> 0 then c
+	  else begin try
+	    List.iter2
+	      (fun t1 t2 ->
+		 let c = compare_tterm t1 t2 in
+		 if c <> 0 then raise (Out c)
+	      ) tl1 tl2; 0 
+	  with Out c -> c end
+
+  let compare_tterm_list tl2 tl1 =
+    let l1 = List.map depth_tterm tl1 in
+    let l2 = List.map depth_tterm tl2 in
+    let l1 = List.rev (List.fast_sort compare l1) in
+    let l2 = List.rev (List.fast_sort compare l2) in
+    let c = try
+      List.iter2
+	(fun n m -> 
+	   if n <> m then raise (Out (n-m))
+	) l1 l2; 0
+    with 
+      | Out c -> c 
+      | _ -> (List.length l2) - (List.length l1) in
+    if c <> 0 then c
+    else begin try
+      List.iter2
+	(fun t1 t2 ->
+	   let c = compare_tterm t1 t2 in
+	   if c <> 0 then raise (Out c)
+	) tl1 tl2; 0 
+    with Out c -> c end
+    
+
+  module ST = Set.Make(
+    struct
+      type t = Why_ptree.tterm
+      let compare = compare_tterm
+    end)
+
+  module SMT = Set.Make(
+    struct
+      type t = ST.t
+      let compare a b = ST.compare b a (*
+	let c = compare_tterm (ST.max_elt b) (ST.max_elt a)  in
+	if c <> 0 then c
+	else let c = (ST.cardinal a) - (ST.cardinal b) in
+	if c <> 0 then c
+	else ST.compare b a *)
+    end)
+
+  let at_most n l =
+    let l = List.fast_sort compare_tterm_list l in
+    (* let st = List.fold_left 
+      (fun s l' -> 
+	 let s' = List.fold_left (fun s'' e -> ST.add e s'') ST.empty l' in
+	 SMT.add s' s
+      ) SMT.empty l in *)
+    let rec atmost acc n l = 
+      match n, l with
+        | n, _ when n <= 0 -> acc
+        | _ , [] -> acc
+        | n, x::l -> 
+            if List.mem x acc then atmost acc n l
+            else atmost (x::acc) (n-1) l
+	
+    in
+    List.rev (atmost [] n l)
   let is_var t = match t.tt_desc with
     | TTvar _ -> true
     | TTapp (_,l) -> l=[]
@@ -600,7 +728,14 @@ module Triggers = struct
 	  if Options.notriggers || qf.qf_triggers=[] then
 	    (make_triggers false (bv',vty') (STRS.elements trs1))@
 	      (make_triggers false (bv',vty') (STRS.elements trs2))
-	  else filter_good_triggers (bv',vty') qf.qf_triggers
+	  else 
+	    begin
+	      let lf = filter_good_triggers (bv',vty') qf.qf_triggers in
+	      if lf<>[] then lf
+	      else
+		(make_triggers false (bv',vty') (STRS.elements trs1))@
+		  (make_triggers false (bv',vty') (STRS.elements trs2))
+	    end
 	in
 
 	let trs = 
@@ -624,7 +759,10 @@ module Triggers = struct
 	let trs' = 
 	  if Options.notriggers || qf.qf_triggers=[] then
 	    make_triggers gopt (bv',vty') (STRS.elements trs)
-	  else filter_good_triggers (bv',vty') qf.qf_triggers
+	  else 
+	    let lf = filter_good_triggers (bv',vty') qf.qf_triggers in
+	    if lf <> [] then lf
+	    else make_triggers gopt (bv',vty') (STRS.elements trs)
 	in
 	let trs = 
 	  STRS.filter (fun (_,bvt,_) -> Sy.is_empty (Sy.inter bvt bv')) trs in
@@ -1061,9 +1199,9 @@ let check_duplicate_params l =
 let type_decl acc d = 
   try
     match d with
-	Logic (loc,ac, ext, lp, ty) -> 
+	Logic (loc,ac, lp, ty) -> 
 	  List.iter (Logics.add (Profile.of_logictype ty) loc ac) lp;
-	  TLogic(loc,ext,lp,ty)::acc
+	  TLogic(loc,lp,ty)::acc
       | Axiom(loc,name,f) -> 
 	  let f,_ = type_form Env.empty f in 
 	  let f = Triggers.make true f loc in
@@ -1071,6 +1209,7 @@ let type_decl acc d =
       | Goal(loc,n,f) -> 
 	  (*let f = move_up f in*)
 	  let env , axioms , goal = 
+	    (*Env.empty, [], f*)
 	    intro_hypothesis Env.empty 
 	      (not (!smtfile or !satmode)) f in
 	  let axioms =
@@ -1093,7 +1232,7 @@ let type_decl acc d =
 	    | _ -> PPredicate l 
 	  in
 	  let l = List.map (fun (_,x,t) -> (x,t)) l in
-	  Logics.add (Profile.of_logictype ty) loc false n; (* TO DO *)
+	  Logics.add (Profile.of_logictype ty) loc Symbols.No n; (* TODO *)
 	  let lvar = List.map (fun (x,_) -> {pp_desc=PPvar x;pp_loc=loc}) l in
 	  let p = {pp_desc=PPapp(n,lvar) ; pp_loc=loc } in
 	  let infix = match d with Function_def _ -> PPeq | _ -> PPiff in
@@ -1105,9 +1244,9 @@ let type_decl acc d =
 	  (match d with 
 	     | Function_def(_,_,_,t,_) -> TFunction_def(loc,n,l,t,f)::acc
 	     | _ ->  TPredicate_def(loc,n,l,f)::acc)
-      | TypeDecl(loc,ext,ls,s) -> 
+      | TypeDecl(loc,ls,s) -> 
 	  Types.add ls s loc;
-	  TTypeDecl(loc,ext,ls,s)::acc
+	  TTypeDecl(loc,ls,s)::acc
   with Warning(e,loc) -> 
     Loc.report loc; printf "Warning: %a@." report e;
     TWarning(loc)::acc
