@@ -1,11 +1,15 @@
 (**************************************************************************)
 (*                                                                        *)
 (*     The Alt-ergo theorem prover                                        *)
-(*     Copyright (C) 2006-2008                                            *)
+(*     Copyright (C) 2006-2010                                            *)
 (*                                                                        *)
 (*     Sylvain Conchon                                                    *)
 (*     Evelyne Contejean                                                  *)
-(*     CNRS-LRI-Universite Paris Sud                                      *)
+(*     Stephane Lescuyer                                                  *)
+(*     Mohamed Iguernelala                                                *)
+(*     Alain Mebsout                                                      *)
+(*                                                                        *)
+(*     CNRS - INRIA - Universite Paris Sud                                *)
 (*                                                                        *)
 (*   This file is distributed under the terms of the CeCILL-C licence     *)
 (*                                                                        *)
@@ -15,12 +19,13 @@ type error
 
 exception Error of error * Why_ptree.loc
 
+(*
 type tdecl = 
   | Assume of Formula.t * Why_ptree.loc * bool
   | PredDef of Formula.t * Why_ptree.loc
   | Query of string * Formula.t * Literal.t list * Why_ptree.loc
-
-val make_cnf : (Why_ptree.typed_decl * bool) list -> tdecl Queue.t
+*)
+val make_cnf : (Why_ptree.typed_decl * bool) list -> Why_ptree.sat_tdecl Queue.t
 
 val file : Why_ptree.file -> Why_ptree.typed_decl list list
 
